from __future__ import absolute_import

import json
import os
import uuid

import matplotlib.pyplot as plt
from .mplexporter.exporter import Exporter
from jinja2 import Environment, PackageLoader

from .leaflet_renderer import LeafletRenderer
from . import maptiles

_attribution = '<a href="https://github.com/jwass/mplleaflet">mplleaflet</a>'

env = Environment(loader=PackageLoader('mplleaflet', 'templates'),
                  trim_blocks=True, lstrip_blocks=True)

def fig_to_html(fig=None, template='base.html', tiles=None, crs=None,
                epsg=None):
    """
    Convert a Matplotlib Figure to a Leaflet map

    Parameters
    ----------
    fig : figure, default gcf()
        Figure used to convert to map
    template : string, default 'base.html'
        The Jinja2 template to use
    tiles : string or tuple
        The tiles argument is used to control the map tile source in the
        Leaflet map. Several simple shortcuts exist: 'osm', 'mapquest open',
        and 'mapbox bright' may be specified to use those tiles.

        The argument may be a tuple of two elements. The first element is the
        tile URL to use in the map's TileLayer, the second argument is the
        attribution to display.  See
        http://leafletjs.com/reference.html#tilelayer for more information on
        formatting the URL.

        See also maptiles.mapbox() for specifying Mapbox tiles based on a
        Mapbox map ID.
    crs : dict, default assumes lon/lat
        pyproj definition of the current figure. If None, then it is assumed
        the plot is longitude, latitude in X, Y.
    epsg : int, default 4326
        The EPSG code of the current plot. This can be used in place of the
        'crs' parameter.

    Note: only one of 'crs' or 'epsg' may be specified. Both may be None, in
    which case the plot is assumed to be longitude / latitude.

    Returns
    -------
    String of html of the resulting webpage

    """
    if tiles is None:
        tiles = maptiles.osm
    elif isinstance(tiles, basestring):
        if tiles not in maptiles.tiles:
            raise ValueError('Unknown tile source "{}"'.format(tiles))
        else:
            tiles = maptiles.tiles[tiles]

    template = env.get_template(template)

    if fig is None:
        fig = plt.gcf()
    dpi = fig.get_dpi()

    renderer = LeafletRenderer(crs=crs, epsg=epsg)
    exporter = Exporter(renderer)
    exporter.run(fig)

    attribution = _attribution + ' | ' + tiles[1]

    mapid = str(uuid.uuid4()).replace('-', '')

    gjdata = json.dumps(renderer.geojson())
    params = {
        'geojson': gjdata,
        'width': fig.get_figwidth()*dpi,
        'height': fig.get_figheight()*dpi,
        'mapid': mapid,
        'tile_url': tiles[0],
        'attribution': attribution,
    }
    html = template.render(params)

    return html


def fig_to_geojson(fig=None, **kwargs):
    """
    Returns a figure's GeoJSON representation as a dictionary

    All arguments passed to fig_to_html()

    Returns
    -------
    GeoJSON dictionary

    """
    if fig is None:
        fig = plt.gcf()
    renderer = LeafletRenderer(**kwargs)
    exporter = Exporter(renderer)
    exporter.run(fig)

    return renderer.geojson()


def save_html(fig=None, fileobj='_map.html', **kwargs):
    if isinstance(fileobj, str):
        fileobj = open(fileobj, 'w')
    if not hasattr(fileobj, 'write'):
        raise ValueError("fileobj should be a filename or a writable file")
    html = fig_to_html(fig, **kwargs)
    fileobj.write(html)
    fileobj.close()


def display(fig=None, closefig=True, **kwargs):
    """
    Convert a Matplotlib Figure to a Leaflet map. Embed in IPython notebook.

    Parameters
    ----------
    fig : figure, default gcf()
        Figure used to convert to map
    closefig : boolean, default True
        Close the current Figure

    """
    from IPython.display import HTML
    if fig is None:
        fig = plt.gcf()
    if closefig:
        plt.close(fig)

    html = fig_to_html(fig, template="ipynb.html", **kwargs)
    return HTML(html)


def show(fig=None, path='_map.html', **kwargs):
    """
    Convert a Matplotlib Figure to a Leaflet map. Open in a browser

    Parameters
    ----------
    fig : figure, default gcf()
        Figure used to convert to map
    path : string, default '_map.html'
        Filename where output html will be saved

    See fig_to_html() for description of keyword args.

    """
    import webbrowser
    fullpath = os.path.abspath(path)
    with open(fullpath, 'w') as f:
        save_html(fig, fileobj=f, **kwargs)
    webbrowser.open('file://' + fullpath)
